#!/usr/bin/env python2
# -*- coding: utf-8 -*-

# Voronoi: http://web.archive.org/web/20161201124949/http://www.saliences.com:80/projects/npr/stippling/options.html
#          https://cs.nyu.edu/~ajsecord/stipples.html
# Older solutions: http://gimpchat.com/viewtopic.php?f=10&t=11690
# Example code by Lylejk: start voronoi.exe -s 8000 -t .2 -n voronoi.png voronoi.svg
#                         start voronoi.exe -p 2 -n -s 3600 -t .2 -z 1 voronoi.png voronoi.svg %1
# Ver 0.1 2020 Initial version.
# Ver 0.2 17/10/2020 - add options: Copy svg file to another directory, internalization: Voronoi Stippler.po

'''
License:
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 3 of the License.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
The GNU Public License is available at
http://www.gnu.org/copyleft/gpl.html


Options voronoi_stippler:

--stipples -s
The number of stipple points to render. Generally speaking, the larger the number of stipples, the more detailed the resulting output image,
at the expense of longer render times. Defaults to 4000.

--threshold -t
The cut-off point for the convergence of the stippled output. The unit of this value is the averaged displacement of all stipple points after an iteration of Llyod’s method.
That is, if the averaged displacement is lower than this value, then the rendering process has finished. 
After each iteration of the rendering process, the expectation is that the stipple points have moved closer to their ideal spot. 
Generally speaking, the lower this value, the less noisy the rendering will be (a desirable quality) at the expense of longer render times. 
As a rule of thumb, the more stipple points there are, the lower this value should be, but lots of fiddling is necessary to get an ideal outcome. Defaults to 0.1.

--no-overlap -n
Guarantees that stipple points will not overlap each other at the expense of some tone preservation. Defaults to OFF.

--sizing-factor -z
The radius of each stipple point will be multiplied by this factor. Use this parameter to control the tone preservation quality of the output image. Defaults to 1.0.

--subpixels -p
Controls the tile size of centroid computations. Theoretically, higher values produce more faithful tone preservation at the expense of rendering speed,
but real world outcomes are highly variable. Defaults to 5.
'''

from gimpfu import *
import sys, os, gettext, time
import shlex
import subprocess
import subprocess as sub
from shutil import copyfile

def sys_file(f_name):
	sys_encoded = sys.getfilesystemencoding()
	if os.name == 'nt': coded = f_name.encode(sys_encoded)
	else: coded = f_name
	return(coded)
	
gettext.install("gimp20", gimp.locale_directory, unicode=True)	
whoiam = os.path.abspath(sys.argv[0])
# initialize internationalisation in a user-locale (by rob_brz1)
locale_directory = os.path.join(os.path.dirname(whoiam), 'locale')
gettext.install( "Voronoi Stippler", locale_directory, unicode=True )
	
def plugin_main(img, layer, save, stipplepointsParam,  displacementParam, nooverlap, sizingfactorParam, subpixParam, typeParam, addbg, scolor, copy_svg, newname, output_back_Folder, timerender):

	if os.name == 'nt':
		voronoi_directory = 'voronoi-stippler.exe'
	else:
		voronoi_directory = '/usr/bin/voronoi_stippler'
	
	outputFolder = gimp.directory + os.sep + "tmp"
	
	if save==0:
		pdb.file_png_save_defaults(img, layer,  "voronoi" + ".png", "raw_filename")		
	if save==1:
		gimp.pdb.file_png_save(img, layer,  "voronoi" + ".png", "raw_filename", 0, 9, 0, 0, 0, 0, 0, run_mode=RUN_INTERACTIVE)

	pdb.gimp_item_set_visible(layer, 0)
	
	
	if nooverlap==1:		# on
		overlap = ""	
	if nooverlap==0:        # off
		overlap = " -n"	

		
	if typeParam == 0:
		params = " -c"
	if typeParam == 1:
		params = " "
		
	command = voronoi_directory + " -s " + str(stipplepointsParam) + " -t " + str(displacementParam) + overlap + " " + "-z" + " " + str(sizingfactorParam) + " " + "-p " + str(subpixParam) +  params + " voronoi.png voronoi.svg"
	if os.name == 'nt':
		child = subprocess.Popen(command, shell=False)
	else:
		child = subprocess.Popen(command, shell=True)

	file = 'voronoi.svg'	
	gimp.progress_init(_("Rendering '") + file + "'...")	
	time.sleep(abs(timerender))	

	gimp.progress_init(_("Opening '") + file + "'...")
	pdb.gimp_vectors_import_from_file(img, file, True, True)
	resolutionx, resolutiony = pdb.gimp_image_get_resolution(img)
	
	fileImage = pdb.file_svg_load(file, file, resolutionx, 0, 0, 2, run_mode=RUN_INTERACTIVE)
	newLayer = gimp.Layer(img, _("Background"), layer.width, layer.height, 0, layer.opacity, layer.mode)
	img.add_layer(newLayer, 0)               
	fileLayer = fileImage.layers[0]

	pdb.gimp_edit_copy(fileLayer)
	pdb.gimp_floating_sel_to_layer(pdb.gimp_edit_paste(layer, True))
	pdb.gimp_image_lower_item_to_bottom(img,newLayer)
	#pdb.gimp_layer_add_alpha(newLayer)
	if addbg==1:	
		pdb.gimp_context_set_background((scolor))
		pdb.gimp_edit_fill (newLayer, FILL_BACKGROUND)
	if addbg==0:			
		pdb.gimp_image_remove_layer(img, newLayer)
	if copy_svg==1:
		backup_tempsvgfilename = sys_file(output_back_Folder + os.sep + newname + ".svg")
		copyfile(file,backup_tempsvgfilename) #back it up.		

	return
register(
	"python_fu_voronoi_stippler",
	_("This plug-ins:\n\n1. Saves (defaults or interactive) active layer as voronoi.png in the bin directory Gimp. \n2. Launches Voronoi.exe (console) - create fille voronoi.svg. \n------------------- wait for the console to close -------------------\n(* Need to increase 'Waiting Time' - when window 'Render Scalable Vector Graphics' opens before closing console!)\n3. Opened voronoi.svg as layer."),	
	"Voronoi Stippler 0.2",
	"MareroQ",
	"GPL",
	"2020",
	"Voronoi Stippler...", 
	"*", 
	[
	(PF_IMAGE, "img", "TheImage", None),
	(PF_DRAWABLE, "layer", "TheDrawable", None),
	(PF_OPTION, "save", _("Save voronoi.png:"), 0,[_("Defaults"), _("Interactive")]),	
	(PF_INT, "stipplepointsParam",  _("Number of stipple points:\nDefaults to -s=4000 "), 4000),
	(PF_FLOAT, "displacementParam",  _("Threshold:\nDefaults to -t=0.1 "), 0.1),
	(PF_BOOL, "nooverlap", _("Overlap ? \nDefaults to -n=OFF"), False),
	#(PF_BOOL, "fixedradius", "Fixed Radius ? \nDefaults to -f=OFF", False),	
	(PF_INT, "sizingfactorParam",  _("Sizing Factor:\nDefaults to -z=1.0 "), 1),		
	(PF_INT, "subpixParam",  _("Subpixels:\nDefaults to -p=5.0 "), 5),	
	(PF_OPTION, "typeParam", _("Dots:"), 0,[_("Colors"), _("Black")]),
	(PF_BOOL, "addbg", _("Add backgrund layer?"), True),	
	(PF_COLOR, "scolor", _("Background color:"), (255, 255, 255,)),
	(PF_BOOL, "copy_svg", _("Copy svg file to another directory?"), False),
	(PF_STRING,   "newname", _("New Name:"), "gimp scripts net"),		
	(PF_DIRNAME, "output_back_Folder", _("Output directory to save backup svg :"), "d:\\Voronoi\\"),		
	(PF_INT, "timerender", _("Waiting Time(*):"), 20)
	],   
	[],
	plugin_main,
	menu="<Image>/Filters/Edge-Detect",	
	domain = ("Voronoi Stippler", locale_directory)	
	)
	


main()
